#ifndef TRXCONST_H
#define TRXCONST_H
#ifndef M_PI
#define M_PI PI
#endif
/** \file
   Some physical constants and macros.
*/
                                      /* */
                                      /* physics constants */
#define TRX_C     2.99792458E10       /* cm/sec */
#define TRX_AMU   931.49432           /* MeV */
#define TRX_MP    938.27231           /* proton mass (MeV) */ 
#define TRX_MN    9.395729E+02        /* neutron mass (MeV) */ 
#define TRX_M0    0.51099906E3        /* electron rest mass (keV) */
#define TRX_ALPHA (1E0/137.0359895)
#define TRX_HBARC 197.327053          /* MeVfm */
#define TRX_E2    (TRX_ALPHA*TRX_HBARC) /* MeVfm */
#define TRX_RYDBERG 13.6056981E-3     /* keV */
#define TRX_A0    0.529177249E-8      /* Bohr radius (cm) e**2/alpha**2/m0 */
#define TRX_RBOHR  TRX_A0
#define TRX_NAVOGADRO 6.0221367E+23
#define TRX_R0    (TRX_E2/TRX_M0*1E3) /* classical electron radius (fm) */ 
#define TRX_THOMSON (TRX_R0*TRX_R0*PI*8E0/3E0) /* thomson cross section */ 

#define TRX_8LN2 ( /*M_LN2*/ 0.6932 * 8E0 )

                                      /* Gy cm mm**2 / MeV, for H2O */
#define TRXDOSE_MEV2GY ( 1.60217733E-19 * 1E11 )
                                      /* (v/c)**2*/
#define TRXBETA2(m0,T)\
  ( 1.0E0 - ( (m0) / ( (m0) + (T) ) ) * ( (m0) / ( (m0) + (T) ) ) )     

#define TRXGAMMABETA2(gamma,beta2,m0,T)\
  (gamma) = ( (T) + (m0) ) / (m0);\
  (beta2) = ( 1.0E0 - 1.0E0 / ( (gamma) * (gamma) ) );     

#endif
