#ifndef TRXEVAL_H
#define TRXEVAL_H
/** \file
   Common definitions for evaluation functions.
*/
/*
  
   Copyright  1992-2009 M.Kraemer, GSI Darmstadt 

   $Log: trxeval.h,v $
   Revision 1.4  2008/02/05 11:19:52  kraemer
   eval rework

   Revision 1.3  2002/11/25 16:20:49  kraemer
   volmode, xsmax

   Revision 1.2  2001/01/15 16:52:30  kraemer
   bug fixes

   Revision 1.1  2001/01/09 14:54:30  kraemer
   *** empty log message ***


*/

#include "m.h"
#include "cmd.h"
#include "trxproj.h"
#include "trxxs.h"
#include "trxvol.h"
#include "trxparticle.h"

                                      /* masks */
#define TRXEVAL_MASKGLOBAL 0xFF000000 /* global (e.g. debug) */
#define TRXEVAL_MASKFUNC   0x00FF0000 /* function specific flags */
#define TRXEVAL_MASKQUAL   0x0000FFFF /* for additional qualifiers */

                                      /* flags */
#define TRXEVAL_DEBUG      0x80000000 /* general check flag */
#define TRXEVAL_CHECK      TRXEVAL_DEBUG  /* general check flag */
#define TRXEVAL_QUIET      0x10000000 /* suppress error messages */
#define TRXEVAL_NOMSG      TRXEVAL_QUIET
                                      /* */
                                      /** \struct STRXEVALLINETRACKSEQ
                                          \brief line track sequence.
                                      */ 
struct STRXEVALLINETRACKSEQ {
   double dvx[4];
   float rdE;
   float rE;
   unsigned long ulPT;
            long  lIA;
};    
                                      /** \struct STRXEVALLINETRACKID
                                          \brief line track id.
                                      */ 
struct STRXEVALLINETRACKID {
   unsigned long ulSq;                /* # of interactions */ 
   struct STRXEVALLINETRACKSEQ *psSq; /* [ ulSq ] */ 
};    
                                      /** \struct STRXEVALLINETRACKTRK
                                          \brief a track.
                                      */ 
struct STRXEVALLINETRACKTRK {
   unsigned long ulID;                /* # of individual particles */ 
   struct STRXEVALLINETRACKID *psID;  /* [ulID] */ 
};    
                                      /** \struct STRXEVALLINETRACK
                                          \brief line track structure.
                                      */ 
struct STRXEVALLINETRACK {
   double dvLimz[2];                  /**< axial limits [cm] */
   struct STRXEVALLINETRACKTRK *psT;  /**< [ulEvt] tracks */         
};
                                      /** \struct STRXEVALADOSE
                                          \brief For evaluation of axial dose.
                                      */ 
struct STRXEVALADOSE {
   unsigned long ulzScale;            /**< lin/log/... */
   int nz;                            /**< # of axial bins */
   double *pdzBin;                    /**< [nz+1] axial bins [cm] */
   double *pdzDoseE;                  /**< [nz] axial energy deposition [keV] */
   unsigned long *pulzDoseI;          /**< [nz] axial ionization deposition */
   unsigned long *pulzCountE;         /**< [nz] axial energy deposition [counts] */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
};
                                      /** \struct STRXEVALSITE
                                          \brief Microdosimetric site.
                                      */ 
struct STRXEVALSITE {
   float dcos, dsin, dzz, drr;        /**< site coordinate info*/
};
                                      /** \struct STRXEVALMDOSE
                                          \brief For evaluation of microdosimetry.
                                      */ 
struct STRXEVALMDOSE {
#define TRXEVAL_PHD 0x00010000        /* request pulse height distr. */                                        
   unsigned long ulFlag;              /**< flag bits */
   unsigned long ulrScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   int nz;                            /**< # of axial bins */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
   double dDiameter, dR, dR2;         /**< dimensions of microdosimetric site */
   double dESum;                      /**< Dose [Gy] */
   unsigned long ulShape;             /**< sphere/cyl. */
   unsigned long ulSite;              /**< total # of sites */
   unsigned long ulThreshold;         /**< threshold # of ionizations */
   unsigned long *pulNormR;           /**< [nr] # of sites with same distance */ 
   unsigned long *pulBreaksR;         /**< [nr] # of "damages" at distance */ 
   unsigned long *pnPhi;              /**< [nr] # of azimuthal positions at distance */ 
   unsigned long ****pulHit;          /**< [nevt][iz][ir][iphi] hit distribution */
   struct STRXEVALSITE ***psSite;     /**< [nz][rr][iphi] site coordinates */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdzBin;                    /**< axial bins [cm] */
};
                                      /** \struct STRXEVALOUT
                                          \brief For "outside" spectra.
                                      */ 
struct STRXEVALOUT {
   unsigned long ulEScale;            /**< lin/log/... */
   int nE, nA;                        /**< # of energy, angular bins */                  
   double dvLimE[2];                  /**< energy limits [keV] */
   double dvLimA[2];                  /**< angular limits [deg] */
   double *pdEBin;                    /**< energy bins [keV] */
   double *pdABin;                    /**< angular bins (cos()) */
   unsigned long **pulSpcAE;          /**< [nA][nE] spectrum */
};
                                      /** \struct STRXEVALRDOSE
                                          \brief For evaluation of radial dose.
                                      */ 
struct STRXEVALRDOSE {
   unsigned long ulrScale;            /**< lin/log/... */
   double dvLimr[2];                  /**< radial bins */
   double dvLimz[2];                  /**< axial bins */
   unsigned long ulCoreI;             /**< # of ionizations < rmin */
   unsigned long ulCoreE;             /**< # of interactions < rmin */            
   double dCoreE;                     /**< energy deposited < rmin */
   int nr;                            /**< # of radial bins */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdRDoseE;                  /**< radial dose distribution */
   unsigned long *pulRDoseI;          /**< radial ionization distribution */
};
                                      /** \struct STRXEVALRZDOSE
                                          \brief For evaluation of radial-axial dose.
                                      */ 
struct STRXEVALRZDOSE {
   unsigned long ulrScale;            /**< lin/log/... */
   unsigned long ulzScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   int nz;                            /**< # of axial bins */
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdzBin;                    /**< axial bins [cm] */
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvLimz[2];                  /**< axial limits [cm] */
   char *pcrGrid;                     /**< list of radial bins [cm] */
   char *pczGrid;                     /**< list of axial bins [cm] */
   double **pdrzDoseE;                /**< [nr][nz] 2-dim energy dose [Gy] */
   unsigned long **pulrzDoseI;        /**< [nr][nz] 2-dim ionization events */
};
                                      /** \struct STRXEVALSPHEREDOSE
                                          \brief For evaluation of spherical dose.
                                      */ 
struct STRXEVALSPHEREDOSE {
   unsigned long ulrScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   unsigned long ulCoreI;             /* # of ionizations < rmin */
   unsigned long ulCoreE;             /* # of interactions < rmin */            
   double dCoreE;
   double dvLimr[2];                  /**< radial limits [cm] */
   double dvCentre[3];                /**< sphere centre coord.s [cm] */				         
   double *pdrBin;                    /**< radial bins [cm] */
   double *pdRDoseE;            /**<  */
   unsigned long *pulRDoseI;
   unsigned long *pulRCountE;
   char *pcrGrid;                     /**< list of radial bins [cm] */
};
                                      /** \struct STRXEVALTESPC
                                          \brief For evaluation of time-energy distribution.
                                      */ 
struct STRXEVALTESPC {
   unsigned long ultScale;            /**< lin/log/... */
   unsigned long ulEScale;            /**< lin/log/... */
   int nE;                            /**< # of energy bins */                          
   int nt;                            /**< # of time bins */                         
   double dvLimt[2];                  /**< time limits [sec] */
   double dvLimE[2];                  /**< energy limits [keV] */
   double *pdtBin;                    /**< [nt+1] time bins [sec]*/
   double *pdEBin;                    /**< [nE+1] energy bins [keV] */
   unsigned long **pulSpctE;          /**< [nt][nE] spectrum */
};
                                      /** \struct STRXEVALVOLESPC
                                          \brief For evaluation of volume-energy distribution.
                                      */ 
struct STRXEVALVOLESPC {
   unsigned long ulEScale;            /**< lin/log/... */
   int nE;                            /**< # of energy bins */                                  
   int nV;                            /**< # of volumes */                                 
   double dvLimE[2];                  /**< energy limits [keV] */
   double *pdEBin;                    /**< [nE+1] energy bins [keV] */
   unsigned long **pulSpcVE;          /* [nV][nE] spectrum */
};
                                      /** \struct STRXEVALCROSSERSVOLVOL
                                          \brief crossers evaluation volume.
                                      */ 
struct STRXEVALCROSSERSVOLVOL {
   unsigned long ulID;                /**<  */
   unsigned long ulVolStart;          /**<  */
   unsigned long ulFromI;             /**<  */
   unsigned long ulFromO;             /**<  */
   double ddEFromI;                   /**<  */
   double ddEFromO;                   /**<  */
};
                                      /** \struct STRXEVALCROSSERSVOL
                                          \brief crossers evaluation volume.
                                      */ 
struct STRXEVALCROSSERSVOL {
   unsigned long ulInsiders;          /**<  */
   unsigned long ulStoppers;          /**<  */
   unsigned long ulStarters;          /**<  */
   unsigned long ulCrossers;          /**<  */
   double ddEInsiders;                /**<  */
   double ddEStoppers;                /**<  */
   double ddEStarters;                /**<  */
   double ddECrossers;                /**<  */
   
   int nVV;                           /**< # of cross volumes */  
   struct STRXEVALCROSSERSVOLVOL *psVV; /**< [nVV] */
};
                                      /** \struct STRXEVALCROSSERS
                                          \brief for evaluation of crossers et al.
                                      */ 
struct STRXEVALCROSSERS {
#define TRXEVAL_ECINC (1024)          /* realloc() increment */
   int mV;                            /**< # of volumes in evaluation */
   struct STRXEVALCROSSERSVOL *psV;   /**< [mV+1] volume descriptions */
};   
                                      /** \struct STRXEVALPOINTTRACK
                                          \brief point track structure.
                                      */ 
struct STRXEVALPOINTTRACK {
#define TRXEVAL_XSMAX (MTRXXS+1)
   FILE *fpTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ];
   char   cTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ][L_tmpnam+1];
   signed char bTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ];
};   
                                      /** \struct STRXEVAL
                                          \brief Evaluation main structure.
                                      */ 
struct STRXEVAL {
   unsigned long ulFlag;              /**< flag bits */
   unsigned long ulEvt;               /**< actual # of events */				      
   double dDensity;                   /**< density [g/cmm**3] */
   char *pcOutFile;                   /**< outfile */
   char *pcUsrStr;                    /**< user string */
   void *pvEval;                      /**< "private" evaluation data */
                                      /**< evaluation function */
   int (*eEval)(struct STRXEVAL *, void *, int, unsigned long, unsigned long);                       
   dynhandle hEval;                   /**< handle for dynamically loaded evaluation */
                                      /* */
   struct STRXEVALADOSE sADose;
   struct STRXEVALMDOSE sMDose;
   struct STRXEVALOUT   sOut;
   struct STRXEVALRDOSE sRDose;
   struct STRXEVALRZDOSE sRZDose;
   struct STRXEVALSPHEREDOSE sSphereDose;
   struct STRXEVALTESPC sTESpc;
   struct STRXEVALVOLESPC sVolESpc;
   struct STRXEVALCROSSERS sCrossers;
   struct STRXEVALLINETRACK sLTrack;
   struct STRXEVALPOINTTRACK sPTrack;
   struct SCMDERR sCE;



#if defined(OLD)
                                      /* for outside spectra */
   unsigned long ulEScale;            /**< lin/log/... */
   int nE, nA;                                     
   double dvLimE[2];
   double dvLimA[2];
   double *pdEBin;
   double *pdABin;
   unsigned long **pulSpcAE;          /**< [nA][nE] spectrum */
#endif
#if defined(OLD)   
                                      /* for time-energy spectra ... */
   unsigned long ultScale;            /**< lin/log/... */
   unsigned long ulEScale;            /**< lin/log/... */
   int nE, nA;                                     
   int nt;                                     
   double dvLimt[2];
   double dvLimE[2];
   double *pdtBin;
   double *pdEBin;                    /**< energy bins [keV] */
   unsigned long **pulSpctE;          /**< [nt][nE] spectrum */
#endif
#if defined(OLD)   
                                      /* for vol-energy spectra ... */
   int nE;                                     
   int nV;                                     
   double dvLimE[2];
   unsigned long ulEScale;            /**< lin/log/... */
   double *pdEBin;                    /**< energy bins [keV] */
   unsigned long **pulSpcVE;          /* [nV][nE] spectrum */
#endif
#if defined(OLD)   
                                      /* for radial dose */
   unsigned long ulCoreI;             /* # of ionizations < rmin */
   unsigned long ulCoreE;             /* # of interactions < rmin */            
   double dCoreE;
   int nr;
   double *pdrBin;
   double *pdRDoseE;
   unsigned long *pulRDoseI;
   unsigned long *pulRCountE;
   char *pcrGrid;   
#endif   
#if defined(OLD)   
   int nr;
   double *pdzBin;
   double *pdrBin;                    /**< radial bins [cm] */
                                      /* for (r,z) dose */
   unsigned long ulzScale;            /* lin/log/... */
   int nz;
   char *pczGrid;   
   double **pdrzDoseE;                /* [r][z] */
   unsigned long **pulrzDoseI;
   unsigned long **pulrzCountE;
#endif   
#if defined(OLD)   
                                      /* for spherical dose */
   unsigned long ulrScale;            /**< lin/log/... */
   int nr;                            /**< # of radial bins */
   double dvCentre[3];				         
   double dvLimr[2];/**<  */
   double *pdRDoseE;                  /**< radial dose distribution */
   unsigned long *pulRDoseI;          /**< radial ionization distribution */
   unsigned long *pulRCountE;
   unsigned long ulCoreI;             /* # of ionizations < rmin */
   unsigned long ulCoreE;             /* # of interactions < rmin */            
   double dCoreE;
   char *pcrGrid;   /**<  */
#endif   
#if defined(OLD)   
                                      /* for point track */
#define TRXEVAL_XSMAX (MTRXXS+1)
   FILE *fpTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ];
   char   cTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ][L_tmpnam+1];
   signed char bTrack[ MTRXPROJ ][ TRXEVAL_XSMAX ];
#endif
#if defined(OLD)   
                                      /* for evaluation of crossers et al */
#define TRXEVAL_ECINC (1024)
   int mVol;
   int *nvEC;                         /* [mVol+1] */  
   struct STRXEVALCROSSERSVOL **psECV;/* [mVol+1][nvEC] */
   struct STRXEVALCROSSERS  *psEC;    /* [mVol+1] */
#endif    
};
                                      /* */
                                      /* evaluation actions */ 
enum { TRXEVAL_SETUP, TRXEVAL_EVENT, TRXEVAL_EOF, TRXEVAL_ERROR, TRXEVAL_CLEANUP, TRXEVAL_INIT, TRXEVAL_INSPECT,};
                                      /* scale */ 
enum { TRXEVAL_LIN=0, TRXEVAL_LOG };
                                      /* shape for microdosim. */ 
enum { TRXEVAL_SPHERE=0, TRXEVAL_CYL };

/*

   General
   -------

   o check proper cleanup of Eval*()-routines


   TRXEvalFree   : ( eval, flag )
                     release resources
                 3 o todo
		   o to complete:
		     o cleanup of special entries 
                   o calls
                     o TRXEvalInit()
		     
   TRXEvalInit   : 
                 2 x to test
                   o calls

   TRXEvalInspect: 
                 1 x to test
                   o calls


*/
                                      /* function prototypes */
int TRXEvalFree   ( struct STRXEVAL *, unsigned long );
int TRXEvalInit   ( struct STRXEVAL *, unsigned long );
int TRXEvalInspect( struct STRXEVAL *, unsigned long );

struct STRXEVAL *_TRXEvalTest( unsigned long );

/*
   TRXEvalPointTrack: 
                    5 x to test
                      o to complete:
                        o OS/2 branch
                      o calls

   TRXEvalLineTrack : 
                    6 o to test (formally)
		      o to complete
		        o description of energies
		        x start point not included
			x colours
			x e- only
			? interactions by symbols extra
                      o calls

   TRXEvalOPAC      :
                        evaluates OPAC stuff 
                      o to do
                      o calls

   TRXEvalOut       : 
                        evaluates energy/angle distribution on exit
                    4 x to test
                      o calls

   TRXEvalRDose     :
                        evaluates radial dose distribution 
                    1 x to test
                      o calls

   TRXEvalADose     : 
                        evaluates axial dose distribution 
                    2 x to test
                      o calls

   TRXEvalMDose     : 
                        evaluates microdosimetric quantities
                    3 x to test
		      o to complete
		        x PHD, <z> etc
			x cleanup        
		        o cylinder
			x log(r) for outer, lin(r) for inner
                      o calls

*/

int TRXEvalOut  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalTESpc( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalRDose( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalADose( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalMDose( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalOPAC ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
/*
int TRXEvalUsr  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
*/
int TRXEvalCrossers  ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalPointTrack( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );
int TRXEvalLineTrack ( struct STRXEVAL *, struct STRXPARTICLEEVT *, int, unsigned long, unsigned long );

int TRXEvalVolESpc( struct STRXEVAL *, struct STRXVOLEVT *, int, unsigned long, unsigned long );

                                      /* error handling */
int TRXEvalError( struct SCMDERR *, unsigned long );

enum { 
   ERRTRXEVAL_SYS=1,
   ERRTRXEVAL_BADADDR,
   ERRTRXEVAL_BADLIMR,
   ERRTRXEVAL_BADFILENAME,
   ERRTRXEVAL_SHELLEXIT,
   ERRTRXEVAL_BADNEVT,
   ERRTRXEVAL_BADLIMZ,
   ERRTRXEVAL_BADLIME,
   ERRTRXEVAL_BADLIMA,
   ERRTRXEVAL_BADLIMT,
   ERRTRXEVAL_BADGRIDR,
   ERRTRXEVAL_BADGRIDZ,
   ERRTRXEVAL_LOADFAILED,
   ERRTRXEVAL_BINDFAILED
};

#endif
