#ifndef TRXPROJ_H
/*
   Various definitions for projectiles

   Copyright  1992-2000 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: trxproj.h,v $
   Revision 1.2  2002/11/25 16:33:55  kraemer
   various name <=> macros

   Revision 1.1  2001/01/09 14:54:36  kraemer
   *** empty log message ***


*/
#define TRXPROJ_H

#include "cmderr.h"
#include "m.h"

struct STRXZA {
   double dZ;                         /* actual charge */
   double dA;                         /* actual mass (=lA) */
   long   lZ;                         /* number of protons */
   long   lA;                         /* number of nucleons */
};

#define TRXZAMATCH(psZA1,psZA2,iRC)\
   (iRC) = 0;\
   if ( (psZA1)->dA != (psZA2)->dA ) {\
      (iRC) = ERRTRXPROJ_MISMATCHMASS;\
   }\
   if ( (psZA1)->lA != (psZA2)->lA ) {\
      (iRC) = ERRTRXPROJ_MISMATCHMASS;\
   }\
   if ( (psZA1)->dZ != (psZA2)->dZ ) {\
      (iRC) = ERRTRXPROJ_MISMATCHCHARGE;\
   }\
   if ( (psZA1)->lZ != (psZA2)->lZ ) {\
      (iRC) = ERRTRXPROJ_MISMATCHCHARGE;\
   }

int TRXZAMatch  ( struct STRXZA *, struct STRXZA *, unsigned long );
int TRXZAInspect( struct STRXZA *, unsigned long );

/*
   --- 
*/
                                      /* structure to describe projectile */
struct STRXPROJ {
   unsigned long ulFlag;
   unsigned long ulType;              /* particle type */
   unsigned long ulSpace;
   char *pcName;                      /* projectile name */
   double dCharge;                    /* explicit charge */
   double dM0;                        /* rest mass [keV] */
   struct STRXZA sZA;
   struct SCMDERR sCE;
};

enum {
   TRXPROJ_NONE=0, 
   TRXPROJ_N=1, TRXPROJ_X, TRXPROJ_ELECTRON, TRXPROJ_POSITRON, TRXPROJ_ION,
   MTRXPROJ
};

enum {
   TRXPROJ_BITN = ( 1L << TRXPROJ_N ), 
   TRXPROJ_BITX = ( 1L << TRXPROJ_X ), 
   TRXPROJ_BITELECTRON  = ( 1L << TRXPROJ_ELECTRON ), 
   TRXPROJ_BITPOSITRON  = ( 1L << TRXPROJ_POSITRON ), 
   TRXPROJ_BITION       = ( 1L << TRXPROJ_ION ), 
};

#define TRXPROJ_NNAME        "n"
#define TRXPROJ_XNAME        "x"
#define TRXPROJ_ELECTRONNAME "e-"
#define TRXPROJ_POSITRONNAME "e+"
#define TRXPROJ_IONNAME      "ion"

                                      /* Barkas formula */
#define BARKAS(Z0,beta2) (Z0) * ( ( 1E0 - exp( -125E0 * sqrt(beta2) * pow( (Z0), -2E0/3E0 ) ) ) )
                                      /* flags */
                                      /* */
                                      /* global flags */
#define TRXPROJ_DEBUG  0x80000000     /* general check flag */
#define TRXPROJ_QUIET  0x10000000     /* suppress error messages */
#define TRXPROJ_NOMSG  TRXPROJ_QUIET

#define TRXPROJ_REFCOPY 0x00000001    /* do not free() */ 
                                      /* function prototypes */
int TRXProjCopy   ( struct STRXPROJ *, struct STRXPROJ *, unsigned long );
int TRXProjDup    ( struct STRXPROJ *, struct STRXPROJ *, unsigned long );
int TRXProjFree   ( struct STRXPROJ *, unsigned long );
int TRXProjFromStr( struct STRXPROJ *, const char *, unsigned long );
int TRXProjFromZA ( struct STRXPROJ *, struct STRXZA *, unsigned long );
int TRXProjInit   ( struct STRXPROJ *, unsigned long );
int TRXProjInspect( struct STRXPROJ *, unsigned long );
int TRXProjIsValid( struct STRXPROJ *, unsigned long );
int TRXProjList   ( struct STRXPROJ *, unsigned long );
int TRXProjMatch  ( struct STRXPROJ *, struct STRXPROJ *, unsigned long );
int TRXProjSpace  ( struct STRXPROJ *, unsigned long );

int TRXZAInspect( struct STRXZA *, unsigned long );
int TRXZAMatch( struct STRXZA *, struct STRXZA *, unsigned long );

struct STRXPROJ *_TRXProjTest( unsigned long );

const char *TRXProjElementFromZ( long );
const char *TRXProjNameFromType( unsigned long );
unsigned long TRXProjTypeFromName( const char *, int );
                                 
int TRXProjError( struct SCMDERR *, unsigned long );

                                      /* */
int TRXCmdProjTest(
                const char *, long *
              );
/*
   --- error codes
*/
enum { ERRTRXPROJ_SYS=1,
       ERRTRXPROJ_BADADDR,
       ERRTRXPROJ_BADELEMENT,
       ERRTRXPROJ_BADCHARGE,
       ERRTRXPROJ_MISMATCHMASS,
       ERRTRXPROJ_MISMATCHCHARGE,
       ERRTRXPROJ_MISMATCHPARTICLE,
       ERRTRXPROJ_MISMATCHNAME,
       ERRTRXPROJ_BADZ,
       ERRTRXPROJ_BADA,
       ERRTRXPROJ_BADNAME,
       ERRTRXPROJ_BADZA,
       ERRTRXPROJ_NOSUCHPROJ,
       ERRTRXPROJ_MISMATCHRESTMASS,
};

#endif
